/*
Copyright (C) 2009 -  Claudio Salvadori 
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef CXMLPARSER_H
#define CXMLPARSER_H

#include "xercesc/sax/ErrorHandler.hpp"
#include "xercesc/sax/SAXParseException.hpp"
#include "xercesc/parsers/XercesDOMParser.hpp"

#include "xercesc/dom/DOMElement.hpp"
#include "xercesc/dom/DOMLocator.hpp"
#include "xercesc/dom/DOMException.hpp"
#include "xercesc/dom/DOMNamedNodeMap.hpp" 
#include "xercesc/util/XMLString.hpp"
#include "xercesc/util/PlatformUtils.hpp"
#include "xercesc/util/TransService.hpp"
#include "xercesc/dom/DOMNodeList.hpp"
#include "xercesc/dom/DOMText.hpp"

#include "XML_enum.h"

#include "DOMTreeErrorReporter.hpp"

#define strX(s) XMLString::transcode(s)

// Define namespace symbols
XERCES_CPP_NAMESPACE_USE; 

/**
 *\file CXmlParser.h
*\date  Created on: 23-mar-2009
 *\author  Author: Claudio Salvadori
 *\brief Class to implement methods to init, parse, process an xml file. Genercic class.
 */

 /**
 *\class CXMLParser
 *\brief Class to implement methods to init, parse, process an xml file. Genercic class.
 */
class CXMLParser {
	/**
	* \brief Method to decode string from XMLCh* to char*
	* \param  s			Xml string (input)
	* \return Decoded string*/
	char* 				XMLStr2CStr(const XMLCh* s);
	
public:
	
	/** \brief CXMLParser contructor*/
	CXMLParser();
	
	/** \brief CXMLParser virtual distructor*/
	~CXMLParser();
	
	/** \brief Initialize the parser
	* \param bValidate		true if is requested the xml file validation (input)
	* \return Returns an xml_parser_ret_value as a result of initialization operation \see  xml_parser_ret_value*/
	xml_parser_ret_value	Init(void);
	
	/** \brief Parse xml file
	* \param  fn		filename (input)
	* \param  doc 	DOC node pointer: this node will be the root of the parsing tree (output)
	* \return Returns an xml_parser_ret_value as a result of parsing operation \see  xml_parser_ret_value*/
	xml_parser_ret_value	Parse(char *fn,DOMNode* &doc);

	/** \brief Check if the actual node corresponds with the required one
	* \param  node 		Actual DOM node pointer (input)
	* \param  targetname	Name of the target DOM node (input)
	* \return Returns true if the actual node is the target node, false otherwise*/
	bool			IsTargetNode(DOMNode* node,char * targetname);
	
	/** \brief Get the child node of curRoot
	* \param  curRoot		Actual DOM node (root) pointer (input)
	* \param  targetname	Name of the target (child) DOM node (input)
	* \param  targetnode	Child DOM node pointer (output)
	* \return Returns true if the child node is the target node, false otherwise*/
	bool			GetChildNode(DOMNode* curRoot,char * targetname,DOMNode* &targetnode);
	
	/** \brief Search a target node inside the tree
	* \param  curRoot		Actual DOM node (root) pointer(input)
	* \param  targetname	Name of the target (child) DOM node (input)
	* \param  targetnode	Child DOM node pointer (output)
	* \return Returns true if target node is found, false otherwise*/
	bool			FindNodeRootat(DOMNode* curRoot, char * targetname, DOMNode* &targetnode);
	
	/** \brief Get the value of the attribute called "attrname" of the current node (DOMNode* node)
	* \param  node		Target node (input)
	* \param  attrname		Attribute name (input)
	* \param  attrvalue		Attribute value (output)
	* \return Returns true the attribute called attrname is found, false otherwise*/
	bool			GetAttribute(DOMNode* node,char * attrname,char* attrvalue);
	
	/** \brief Find the sibling node with ELEMENT_NODE (=1) type
	* \param  node			DOMNode pointer (input)
	* \return Returns a pointer of DOMNode class of the next sibling node, if it exists;
	*         NULL if it doesn't exist. This function is very usefull because a lot of parsers 
	*         treat empty white-spaces or new lines as text nodes; in this way is possible to 
	*	find only the ELEMENT_NODE*/
	DOMNode*		Get_NextSibling(DOMNode *node);
	
	/** \brief Get the text of the child text node of actual node (DOMNode* node)
	* \param  node		Actual node (input)
	* \param  text		Text (output)
	* \return Returns true if the text node is found, false otherwise*/
	bool 			GetText(DOMNode* node,char* text);
	
	/** \brief Change string encoder
	* \param  codepage	Encoding type (input)*/
	void 			setTranscoder(char *codepage);

protected:

	XercesDOMParser 		*mParser;		/**<XercesDOMParser class object*/
	DOMTreeErrorReporter	 	*mErrorHandler; 	/**< DOMTreeErrorReporter class object*/
	bool 				g_bTranscoder;		/**<brief false if standard encoding (UTF-16) is used, true otherwise*/
	XMLTranscoder 			*g_Transcoder;		/**<brief Transcoder pointer*/
};
#endif
